deferred class LIST[E]

inherit SEQUENCE[E]

feature {NONE} -- Creation

	make_list is
	deferred
	end

feature -- Iterator

	from_iterator (iter: ITERATOR[E]) is
	do
		make_list
		from
			iter.start
		until
			iter.is_off
		loop
			add_last (iter.item)
			iter.next
		end
	end

feature -- Operations

	put (e: E; i: INTEGER) is
	require
		valid_index (i)
	deferred
	ensure
		item (i) = e
	end
	
	add (e: E; i: INTEGER) is
	require
		valid_index (i) or i = upper + 1
	deferred
	ensure
		item (i) = e
		count = old count + 1
	end

	swap (i, j: INTEGER) is
	require
		valid_index (i) and valid_index (j)
	local
		e: E
	do
		e := item(i)
		put (item (j), i)
		put (e, j)
	end

	add_last (e: E) is
	do
		add (e, upper+1)
	ensure
		item (upper) = e
		count = old count + 1		
	end

	reorder (e: E; index: INTEGER) is
	require
		has (e)
		valid_index (index)
	local
		i: INTEGER
	do
		from
			i := index_of (e).min (index)
		until
			i = index_of (e).max (index)
		loop
			swap (i, i + 1)
			i := i + 1
		end
	end
	
feature -- Removing

	remove (i: INTEGER) is
	require
		valid_index (i)
	deferred
	ensure
		count = old count - 1
	end

	clear is
	deferred
	ensure
		is_empty
	end

end -- class LIST[E]
