deferred class SEQUENCE[E]

inherit SAFE_EQUAL[E]

feature -- Deferred features

	count: INTEGER is
	deferred
	ensure
		count >= 0
	end

	item, infix "@" (i: INTEGER): E is
	require
		valid_index (i)
	deferred
	end

feature -- Indexing

	lower: INTEGER is 1
	
	upper: INTEGER is
	do
		Result := count
	end
	
	valid_index(index: INTEGER): BOOLEAN is
	do
		Result := lower <= index and index <= upper
	end

	is_empty: BOOLEAN is
	do
		Result := count = 0
	end

feature -- Accessing:

	first: like item is
	do
		Result := item (1)
	end

	last: like item is
	do
		Result := item (count)
	end

feature -- Looking and Searching:

	has(x: like item): BOOLEAN is
	do
		Result := index_of (x) <= count
	end

	fast_has(x: like item): BOOLEAN is
	do
		Result := fast_index_of (x) <= count
	end

	index_of(element: like item): INTEGER is
	do
		from
			Result := 1
		until
			Result > count or else safe_equal(item (Result),element)
		loop
			Result := Result + 1
		end
	end

	fast_index_of(element: like item): INTEGER is
	do
		from
			Result := 1
		until
			Result > count or item (Result) = element
		loop
			Result := Result + 1
		end
	end

feature -- Agents based features:

	for_all(test: PREDICATE[ANY,TUPLE[E]]): BOOLEAN is
	do
		Result := get_new_iterator.for_all (test)
	end

	exists(test: PREDICATE[ANY,TUPLE[E]]): BOOLEAN is
	do
		Result := get_new_iterator.exists (test)
	end

	do_all(action: PROCEDURE[ANY,TUPLE[E]]) is
	do
		get_new_iterator.do_all (action)
	end

feature -- Other features:

	get_new_iterator: ITERATOR_ON_SEQUENCE[E] is
	do
		create Result.make (Current)
	end

end -- class SEQUENCE[E]
